import logging
import sys
import pathlib
import os
import yaml
import subprocess

logger = logging.getLogger(__name__)


def setup_logging(level, logger_name=__name__):
    _logger = logging.getLogger(logger_name)
    _logger.setLevel(level)
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.setFormatter(logging.Formatter("%(asctime)s — %(name)s — %(levelname)s — %(message)s"))
    _logger.addHandler(console_handler)

    _logger.debug("====[debug mode enabled]====")


def settings():
    settings_file = os.path.join(str(pathlib.Path.home()), ".pyreleaser_io.yaml")
    if os.path.isfile(settings_file):
        logger.debug("parsing: %s", settings_file)
        with open(settings_file, 'r') as f:
            data = yaml.safe_load(f.read())
    else:
        logging.warning(f"skipping missing settings file: {settings_file}")
        data = {}

    return data.get("pyreleaser_io", {})


def run(cmd, check=True):
    """
    run a command, return output
    cmd - command to run (array)

    """
    completed_process = subprocess.run(cmd, shell=True, capture_output=True, check=check)

    return completed_process.stdout.decode().strip()


import os
import logging
from jinja2 import Environment, PackageLoader, select_autoescape

env = Environment(
    loader=PackageLoader('pyreleaser_io', 'res/templates')
)

logger = logging.getLogger(__name__)


def render(template_filename, **kwargs):
    template = env.get_template(template_filename)
    return template.render(kwargs)


def render_to_file(template_filename, target_filename=None, target_dir=None, **kwargs):
    _target_filename = target_filename or \
                      os.path.join(target_dir, template_filename.replace(".jinja2", ""))
    logger.debug(f"writing to {target_filename}")
    with open(_target_filename, 'w') as f:
        logger.debug(f"writing project file: {_target_filename} (args: {kwargs})")
        f.write(render(template_filename, **kwargs))



import logging
try:
    from github import Github, BadCredentialsException
except ModuleNotFoundError:
    logging.error("github module not avaiable, please install pygithub")
    github = None

logger = logging.getLogger(__name__)


def get_instance(gh_settings):
    driver = None
    if gh_settings:
        token = gh_settings.get("token", False)
        if not token:
            raise RuntimeError("GitHub token not available! - check your settings file")

        # attempt to use the driver - if our token was bad this will blow up
        try:
            g = Github(token)
            g.get_user().id
            driver = g
        except BadCredentialsException as e:
            logger.warning(f"github login error: {e.data}")

    else:
        logging.warning("no github credentials available in settings file...")

    return driver


def add_user_info(project, g):
    user = g.get_user()
    project["author"] = user.name

    # always blank for some reason...
    project["author_email"] = user.email


def project_url(project, g):
    return f"https://github.com/{g.get_user().login}/{project.get('name')}.git"



import argparse
import logging
import pyreleaser_io.util
import pkg_resources
import pyreleaser_io.create
import pyreleaser_io.util

logger = logging.getLogger(__name__)


def version():
    return pkg_resources.require("pyreleaser_io")[0].version


def main():
    parser = argparse.ArgumentParser(description="""
    pyreleaser - python release helper tool
    """)

    parser.add_argument('--debug',
                        default=False,
                        action='store_true',
                        help='debug mode enabled')
    parser.add_argument('--version',
                        default=False,
                        action='store_true',
                        help='Print the sysdef version and exit')
    parser.add_argument('--create-project',
                        default=False,
                        action='store_true',
                        help='create a project')
    args = parser.parse_args()
    pyreleaser_io.util.setup_logging(
        logging.DEBUG if args.debug else logging.INFO, "pyreleaser_io"
    )

    settings = pyreleaser_io.util.settings()

    if args.version:
        print(version())
    elif args.create_project:
        pyreleaser_io.create.interactive(settings)
    else:
        parser.print_usage()



import logging
import pyreleaser_io.vcs.github
import pyreleaser_io.vcs.missing
import pyreleaser_io.template
import pyreleaser_io.util
import os

logger = logging.getLogger(__name__)

layouts = {
    "simple": [
        "MANIFEST.in.jinja2",
        "README.md.jinja2",
        "setup.py.jinja2"
    ]
}

vcs = {
    "github": pyreleaser_io.vcs.github,
    "none": pyreleaser_io.vcs.missing
}


def get_vcs(settings):
    driver = False
    _module = None
    for name, module in vcs.items():
        logger.debug(f"trying to setup VCS {name}")
        _module = module
        driver = module.get_instance(settings.get(name, {}))
        if driver:
            logger.debug(f"got a VCS driver! {name}")
            break

    return _module, driver


def interactive(settings):
    vcs_module, vcs_driver = get_vcs(settings)

    print("Lets make something...")
    project = {}
    project["name"] = input("project name? ")

    default_project_url = vcs_module.project_url(project, vcs_driver)
    default_description = "I'll add this later..."
    project["url"] = input(f"Project URL? [{default_project_url}]") or default_project_url
    project["description"] = input(f"Project description? [{default_description}]") or default_description

    vcs_module.add_user_info(project, vcs_driver)

    create_project(project)


def simple_skeleton(project):
    project_name = project.get("name")
    project_src = os.path.join(project_name, project_name)
    init_py = os.path.join(project_src, "__init__.py")

    os.mkdir(project_name)
    for template_file in layouts.get("simple"):
        pyreleaser_io.template.render_to_file(
            template_file,
            target_dir=project_name,
            project=project
        )
    os.mkdir(project_src)
    open(init_py, "a").close()

    pwd = os.getcwd()
    os.chdir(project_name)
    pyreleaser_io.util.run("pipenv install -e .")
    os.chdir(pwd)

    print("all done!")


def create_project(project):
    project_name = project.get("name")
    if os.path.isdir(project_name):
        logger.error(f"Already exists: {project_name}")
    else:
        logger.info("Creating project...")
        simple_skeleton(project)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyreleaser_io",
    version="0.0.2",
    author="Geoff Williams",
    author_email="geoff@declarativesystems.com",
    description="create, test, publish your python projects!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/geoffwilliams/pyreleaser",
    packages=setuptools.find_packages(),
    # pick from https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 1 - Planning",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    entry_points={
        "console_scripts": [
            "pyreleaser=pyreleaser_io.cli:main",
        ]
    },
    include_package_data=True
)






def get_instance(settings):
    return None


def add_user_info(project, g):
    project["author"] = "changeme"
    project["author_email"] = "changeme"


def project_url(project, g):
    return "changeme"
